// Copyright QUANTOWER LLC. © 2017-2021. All rights reserved.

using System;
using System.Drawing;
using System.Linq;
using TradingPlatform.BusinessLayer;

namespace TR_Indicator
{
    /// <summary>
    /// An example of blank indicator. Add your code, compile it and use on the charts in the assigned trading terminal.
    /// Information about API you can find here: http://api.quantower.com
    /// Code samples: https://github.com/Quantower/Examples
    /// </summary>
	public class TR_Indicator : Indicator
    {
        [InputParameter("N")]
        public int n = 100;

        [InputParameter("N1")]
        public int n1 = 100;

        private HistoricalDataCustom dmp;
        private HistoricalDataCustom dmm;
        private HistoricalDataCustom tr;
        private HistoricalDataCustom dmp_ema_data;
        private HistoricalDataCustom dmm_ema_data;
        private HistoricalDataCustom tr_ema_data;

        private Indicator dmp_ema_indi;
        private Indicator dmm_ema_indi;
        private Indicator tr_ema_indi;
        private Indicator dmpa;
        private Indicator dmma;
        private Indicator tra;


        /// <summary>
        /// Indicator's constructor. Contains general information: name, description, LineSeries etc. 
        /// </summary>
        public TR_Indicator() : base()
        {
            // Defines indicator's name and description.
            Name = "TR_Indicator";
            Description = "Trend Rider Indicator";


            AddLineSeries("Low", Color.Transparent);
            AddLineSeries("High", Color.Transparent);
            

            // By default indicator will be applied on main window of the chart
            SeparateWindow = false;
        }

        public TR_Indicator(int n, int n1) : this()
        {
            this.n = n;
            this.n1 = n1;
        }


        /// <summary>
        /// This function will be called after creating an indicator as well as after its input params reset or chart (symbol or timeframe) updates.
        /// </summary>
        protected override void OnInit()
        {
            dmp = new HistoricalDataCustom(this);
            dmm = new HistoricalDataCustom(this);
            tr = new HistoricalDataCustom(this);
            dmp_ema_data = new HistoricalDataCustom(this);
            dmm_ema_data = new HistoricalDataCustom(this);
            tr_ema_data = new HistoricalDataCustom(this);

            dmp_ema_indi = Core.Indicators.BuiltIn.EMA(n, PriceType.Close);
            dmm_ema_indi = Core.Indicators.BuiltIn.EMA(n, PriceType.Close);
            tr_ema_indi = Core.Indicators.BuiltIn.EMA(n, PriceType.Close);
            dmpa = Core.Indicators.BuiltIn.EMA(3, PriceType.Close);
            dmma = Core.Indicators.BuiltIn.EMA(3, PriceType.Close);
            tra = Core.Indicators.BuiltIn.EMA(3, PriceType.Close);

            dmp.AddIndicator(dmp_ema_indi);
            dmm.AddIndicator(dmm_ema_indi);
            tr.AddIndicator(tr_ema_indi);
            dmp_ema_data.AddIndicator(dmpa);
            dmm_ema_data.AddIndicator(dmma);
            tr_ema_data.AddIndicator(tra);
        }

        /// <summary>
        /// Calculation entry point. This function is called when a price data updates. 
        /// Will be runing under the HistoricalBar mode during history loading. 
        /// Under NewTick during realtime. 
        /// Under NewBar if start of the new bar is required.
        /// </summary>
        /// <param name="args">Provides data of updating reason and incoming price.</param>
        protected override void OnUpdate(UpdateArgs args)
        {
            if (Count <= Math.Max(n, n1))
                return;

            double hl = High() - Low();
            double hc = High() - Close(1);
            double cl = Close(1) - Low();  
            double trt1 = Math.Max(hl, hc);
            double tr = Math.Max(trt1, cl);
            double x = High() - High(1);
            double y = Low(1) - Low();
            double dmp = x > y ? x : 0;
            double dmm = x < y ? y : 0;

            this.dmp[PriceType.Close] = dmp;
            this.dmm[PriceType.Close] = dmm;
            this.tr[PriceType.Close] = tr;
            this.dmp_ema_data[PriceType.Close] = dmp_ema_indi.GetValue();
            this.dmm_ema_data[PriceType.Close] = dmm_ema_indi.GetValue();
            this.tr_ema_data[PriceType.Close] = tr_ema_indi.GetValue();

            double dmpa = this.dmpa.GetValue();
            double dmma = this.dmma.GetValue();
            double tra = this.tra.GetValue();
            double dip = dmpa / tra * 100;
            double dim = dmma / tra * 100;
            double ub = Enumerable.Range(1, n1).Max(i => High(i));
            double lb = Enumerable.Range(1, n1).Min(i => Low(i));
            double mb = (ub + lb) / 2;
            bool b = (dip > dim) && (Close() > mb);
            bool s = (dim > dip) && (Close() < mb);

            SetValue(Low(), 0);
            SetValue(High(), 1);

            LinesSeries[0].SetMarker(0, b ? new IndicatorLineMarker(Color.Green, bottomIcon: IndicatorLineMarkerIconType.UpArrow) : null);
     
            LinesSeries[1].SetMarker(0, s ? new IndicatorLineMarker(Color.Red, upperIcon: IndicatorLineMarkerIconType.DownArrow) : null);





        }


    }
}
